/* sumsiz -- give total size of a list of files
   Copyright (C) 1998 Ben Russo

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/* Written by Ben Russo  <Ben.Russo@GlobalOne.net>
      (( I am getting a new job soon, and don't know what my e-mail may be ))
   So you may try        <Ben.Russo@unitedmessaging.com>
   Or 			 <Ben@unitedmessaging.com>		 */

/*   sumsiz.c   by Ben Russo Dec 23, 1998

  This program will accept input on standard in that is in the same
  format of the output of an "ls -l" command.

  The purpose is to report the total size of files that are in a listing.

  You might redirect output from another command for example:

	find /home -mtime -10 -exec ls -l {} \; | sumsiz

  This might output 	"TOTAL SIZE OF FILES = 4208 B"
  Or: 			"TOTAL SIZE OF FILES = 11 MB  962796 B"  

##################
#################
###   BUGS BUGS BUGS
#################
##################
  There are bugs inherent to the way this program works!!!
  For example, if one of the directories in /home was a link to one of the
   other directories in home, then the example given above would count
   the size of all the files in that directory TWICE.
  If there were 2 links to that directory, then the size of said files 
   would be counted 3 times.

   I have been a sysadmin of a very large UNIX shop for a long time, and 
   have found that I often use "find" and all it's options for 
   troubleshooting, monitoring and maintenance.
   
   Other than complicated and inefficient shell scripts using awk or cut
   and doing integer math that often ran into problems, I never found a 
   simple way to figure out the total size of whatever group of files
   I was "find"ing was.

   A "real" C programmer will probably look at this program and think of 
   a gazillion things to improve and fix.  Ideally I think that the 
   functionality of this prog should be built into the "find" command.

*/



#include<stdio.h>
#include<string.h>
#include<stdlib.h>
#include<unistd.h>
#include<errno.h>
#include<ctype.h>

char buffer[2176] = "\0";
unsigned long sumbytsize=0;
unsigned long bytsize=0;
int megs=0;
int tmpmegs=0; 
static long int MEG = 1048576;

int glin(char *tmpbuffer)
{
 char tmpc[2];
 tmpc[1] = '\0';
 buffer[0] = '\0';
 tmpc[0] = getchar();

 if (tmpc[0] != '-')
   {
    while ( (tmpc[0] != '\n') && (tmpc[0] != EOF) )
       tmpc[0] = getchar();
    return(tmpc[0]);
   }
 
 while ( ( tmpc[0] != EOF ) && ( tmpc[0] != '\n' ) )
   {
   strcat(tmpbuffer, tmpc); 
   tmpc[0] = getchar();
   }
 strcat(tmpbuffer, "\n\0");
 return(tmpc[0]);
}

unsigned long int gsiz()
{
char number[15] = "\0";
int i = 0;
/* printf("\t\tentering gsiz()\n\n"); */
for (i; i < 12; i++)
 {
/* printf("\t\tin for loop to get bytsize out of buffer i = %d\n\n",i); */
 if ( (isdigit(buffer[33+i])) || (isblank(buffer[33+i])) )
  {
   number[i] = buffer[33+i];
   number[i+1] = '\0';
  }
  else
   i = 128;
  }
return(strtoul(number, NULL, 0));   
}


int main()
{
int rval = 1;

while ( rval != EOF )
   {
/*   printf("\t\tmade it to while loop!\n\n"); */
   rval = glin(buffer);
/*    printf(buffer); */
   if (buffer[0] == '-')
       bytsize = 0;
       bytsize = gsiz();
/*    printf("\t\trval = %d, bytsize = %u\n\n", rval, bytsize); */
   if (( bytsize > 0 ) && ( rval != EOF ))
      {
      sumbytsize = sumbytsize + bytsize;
      if (sumbytsize >= MEG)
        {
        tmpmegs = sumbytsize / MEG; 
        sumbytsize = sumbytsize % MEG;
        megs = tmpmegs + megs;
        }
      }
/*   printf("\t\tSubtotal SIZE OF FILES = %u MB  %u B \n",megs,sumbytsize); */
   }
if (megs > 0)
   printf("\t\tTOTAL SIZE OF FILES = %u MB  %u B \n",megs,sumbytsize);
else
  printf("\t\tTOTAL SIZE OF FILES = %u B \n",sumbytsize);
return(0);
}
